package com.muyu.etl.data.access.data.access.client.mysql;

import com.muyu.common.core.domain.Result;
import com.muyu.etl.data.access.data.access.client.DataSourcePool;
import com.muyu.etl.data.access.data.access.client.basic.BaseDataAbsSource;
import com.muyu.source.core.DataType;
import com.muyu.source.core.DataValue;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Author: DongZeLiang
 * @date: 2024/8/28
 * @Description: mysql数据源
 * @Version: 1.0
 */
@Log4j2
@Service("mysql-data-source")
public class MySqlDataSource extends BaseDataAbsSource {

    private final DataSourcePool dataSourcePool;

    public MySqlDataSource (DataSourcePool dataSourcePool) {
        super();
        this.dataSourcePool = dataSourcePool;
    }

    /**
     * 获取查询条数
     * @return
     */
    public Result getCount(){
        MySqlQuery query = getQuery();
        HikariDataSource hikariDataSource = dataSourcePool.getDataSource(query.getDataSourceId());
        try(Connection conn = hikariDataSource.getConnection();
        Statement statement = conn.createStatement();) {

            ResultSet resultSet = statement.executeQuery(query.getSql());
            if (resultSet.next()) {
                return Result.success(resultSet.getInt(1));
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return Result.success(0);
    }

    /**
     * 添加数据库
     * @return
     */
    public Result addTargetDatabase(){
        MySqlQuery query = getQuery();
        HikariDataSource hikariDataSource = dataSourcePool.getDataSource(query.getDataSourceId());
        try(Connection conn = hikariDataSource.getConnection();
            PreparedStatement preparedStatement = conn.prepareStatement(query.getSql());) {

            return Result.success(preparedStatement.executeUpdate());
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取数据值
     * @return
     */
    @Override
    public DataValue getDataStructure () {
        MySqlQuery query = getQuery();
        Connection connection = null;
        String sql = query.getSql();
        Map<String, Object> queryParams = query.getParams();
        log.info(sql);
        log.info(queryParams);

        try {
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            ResultSet resultSet = preparedStatement.getResultSet();
            if(resultSet.next()){
                DataValue.builder()
                        .key(resultSet.getCursorName())
                        .label("")
                        .value(resultSet.getObject(resultSet.getCursorName(), String.class))
                        .type(DataType.STRING);

            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return null;
    }

    /**
     * 获取数据记录
     * @return
     */
    @Override
    public DataValue[] getRow () {
        return new DataValue[0];
    }

    /**
     * 获取数据集
     * @return
     */
    @Override
    public DataValue[][] getRows () {
        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();
        MySqlQuery query = getQuery();
        //初始化一个列表,用于存储数据值对象
        DataValue[][] rows = null;

        HikariDataSource dataSource = dataSourcePool.getDataSource(query.getDataSourceId());

        Connection conn = dataSourcePool.getConnection(dataSource);
        try {

            Statement statement = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);
            //准备SQL查询
            ResultSet resultSet = statement.executeQuery(query.getSql());
            int row = 0;
            // 将游标移动到最后行
            if (resultSet.last()) {
                // 获取行数
                row = resultSet.getRow();
                log.info("需要处理数据--->{}条",row);
                // 如果需要重置游标以便后续处理
                resultSet.beforeFirst();
            } else {
                System.out.println("没有数据");
            }
            //获取元数据
            ResultSetMetaData metaData = resultSet.getMetaData();
            //获取列的数量
            int columnCount = metaData.getColumnCount();
            rows = new DataValue[row][columnCount];
            //遍历每一行数据
            while (resultSet.next()) {
                // 提交任务给线程池
                for (int i = 1; i <= columnCount; i++) {
                    if (resultSet.isFirst()){
                        String columnTypeName = metaData.getColumnTypeName(i);
                        DatabaseMetaData metaDataTwo = conn.getMetaData();
                        ResultSet columns = metaDataTwo.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                        String remarks = null;
                        while (columns.next()) {
                            remarks = columns.getString("REMARKS");
                        }
                        DataValue dataStructure = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.getTargetClass(columnTypeName)))
                                .type(DataType.getDataType(columnTypeName))
                                .build();
                        int currentRow = resultSet.getRow() - 1; // 因为行索引从1开始
                        rows[currentRow][i - 1] = dataStructure;
                        map.put(i, dataStructure);
                    }else {
                        DataValue dataStructure = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(map.get(i).getLabel())
                                .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                                .type(map.get(i).getType())
                                .build();
                        int currentRow = resultSet.getRow() - 1; // 因为行索引从1开始
                        rows[currentRow][i - 1] = dataStructure;
                    }
                }
            }

        }catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
            dataSource.evictConnection(conn);
        }
        log.info("查询结果数量为：rows == > {}", rows.length);
        return rows;
    }
}
