package com.muyu.source.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.source.domain.AssetAuthorization;
import com.muyu.source.domain.DataSource;
import com.muyu.source.domain.Structure;
import com.muyu.source.domain.TableInfo;
import com.muyu.source.mapper.DataSourceMapper;
import com.muyu.source.pool.MysqlPool;
import com.muyu.source.service.*;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import static com.muyu.source.pool.config.BaseConfig.SELECTALL;
import static com.muyu.source.pool.config.BaseConfig.SELECTFIELD;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-20-20:06
 * @ Version：1.0
 * @ Description：数据源业务层
 */
@Service
public class DataSourceServiceImpl extends ServiceImpl<DataSourceMapper, DataSource> implements DataSourceService {

    @Resource
    private DataTypeService dataTypeService;
    @Resource
    private AssetDataSourceService assetDataSourceService;
    @Resource
    private ChildrenService childrenService;
    @Resource
    private TableDataService tableDataService;
    @Resource
    private TableInfoService tableInfoService;
    @Resource
    private StructureService structureService;
    @Resource
    private AssetAuthorizationService assetAuthorizationService;

    private static final Long PARENTID = 0L;
    private static final String DRIVERNAME = "com.mysql.cj.jdbc.Driver";
    private static final String MYSQL = "MySql";
    private static final String REDIS = "redis";


    /**
     * 精确查询数据源
     *
     * @param id 数据源主键
     * @return 数据源
     */
    @Override
    public DataSource selectDataSourceById(Long id) {
        LambdaQueryWrapper<DataSource> queryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(id, "id不可为空");
        queryWrapper.eq(DataSource::getId, id);
        return this.getOne(queryWrapper);
    }


    /**
     * 查询数据源列表
     *
     * @param dataSource 数据源
     * @return 数据源
     */
    @Override
    public List<DataSource> selectDataSourceList(DataSource dataSource) {
        LambdaQueryWrapper<DataSource> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(dataSource.getName())) {
            queryWrapper.like(DataSource::getName, dataSource.getName());
        }
        if (StringUtils.isNotEmpty(dataSource.getSystemName())) {
            queryWrapper.like(DataSource::getSystemName, dataSource.getSystemName());
        }
        if (StringUtils.isNotEmpty(dataSource.getDataType())) {
            queryWrapper.eq(DataSource::getDataType, dataSource.getDataType());
        }
        if (StringUtils.isNotEmpty(dataSource.getIp())) {
            queryWrapper.like(DataSource::getIp, dataSource.getIp());
        }
        if (StringUtils.isNotEmpty(dataSource.getPort())) {
            queryWrapper.like(DataSource::getPort, dataSource.getPort());
        }
        return this.list(queryWrapper);
    }

    /**
     * 唯一 判断
     *
     * @param dataSource 数据源
     * @return 数据源
     */
    @Override
    public Boolean checkIdUnique(DataSource dataSource) {
        LambdaQueryWrapper<DataSource> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(DataSource::getId, dataSource.getId());
        return this.count(queryWrapper) > 0;
    }

    /**
     * 测试连接
     *
     * @param etlDataScore 数据源
     * @return 结果
     */
    @Override
    public Boolean testConnection(DataSource etlDataScore) {
        if (etlDataScore.getDataType().equals(MYSQL)) {
            MysqlPool mysqlPool = new MysqlPool(etlDataScore);
            mysqlPool.init();
            Connection conn = mysqlPool.getConn();
            mysqlPool.replease(conn);
            mysqlPool.closeConn();
        }
        return true;
    }

    /**
     * 同步资产结构
     *
     * @param etlDataScore
     * @return
     */
    @Override
    public Integer syncAssetStructure(DataSource etlDataScore) {

        try {
            if (etlDataScore.getDataType().equals(MYSQL)) {
                MysqlPool mysqlPool = new MysqlPool(etlDataScore);
                mysqlPool.init();
                Connection conn = mysqlPool.getConn();
                TableInfo tableInfoServiceOne = tableInfoService.getOne(new LambdaQueryWrapper<>() {{
                    eq(TableInfo::getBasicId, etlDataScore.getId())
                            .last("limit 1");
                }});
                //如果存在就删除
                if (StringUtils.isNotNull(tableInfoServiceOne)) {
                    List<TableInfo> list = tableInfoService.list(new LambdaQueryWrapper<>() {{
                        eq(TableInfo::getBasicId, etlDataScore.getId());
                    }});
                    list.forEach(tableInfo -> {
                        structureService.remove(new LambdaQueryWrapper<>() {{
                            eq(Structure::getTableId, tableInfo.getId());
                        }});
                    });
                    tableInfoService.remove(new LambdaQueryWrapper<>() {{
                        eq(TableInfo::getBasicId, etlDataScore.getId());
                    }});
                    assetAuthorizationService.remove(new LambdaQueryWrapper<>() {{
                        eq(AssetAuthorization::getTableId, tableInfoServiceOne.getId())
                                .or()
                                .eq(AssetAuthorization::getBasicId, etlDataScore.getId());
                    }});
                }

                TableInfo tableInfoInsert = TableInfo.builder()
                        .basicId(etlDataScore.getId())
                        .parentId(PARENTID)
                        .tableRemark("")
                        .center("Y")
                        .type("dataSource")
                        .tableName(etlDataScore.getDatabaseName())
                        .createBy(SecurityUtils.getUsername())
                        .createTime(new Date())
                        .build();

                //添加数据库table_info表
                tableInfoService.saveOrUpdate(tableInfoInsert, new LambdaUpdateWrapper<TableInfo>(TableInfo.class) {{
                    eq(TableInfo::getTableName, tableInfoInsert.getTableName());
                    eq(TableInfo::getBasicId, etlDataScore.getId());
                }});

                //根据数据库id和数据库名称或表名称查询
                TableInfo tableInfo = tableInfoService.selectTableInfoByName(tableInfoInsert);

                DatabaseMetaData metaData = conn.getMetaData();

                ResultSet rs = metaData.getTables(etlDataScore.getDatabaseName(), null, "%", new String[]{"TABLE"});

                while (rs.next()) {
                    String tableName = rs.getString("TABLE_NAME");
                    String tableRemark = rs.getString("REMARKS");

                    PreparedStatement preparedStatement = conn.prepareStatement(SELECTALL + tableName+" limit 1");

                    ResultSet resultSet = preparedStatement.executeQuery();

                    Long rowCount = 0L;
                    while (resultSet.next()) {
                        rowCount++;
                    }
                    TableInfo build = TableInfo.builder()
                            .basicId(etlDataScore.getId())
                            .tableName(tableName)
                            //bug点，tableRemark为空，造成空指针异常
                            .tableRemark(tableRemark == null ? "" : tableRemark)
                            .parentId(tableInfo.getId())
                            .type("dataTable")
                            .center("Y")
                            .updateBy(SecurityUtils.getUsername())
                            .dataNum(rowCount)
                            .updateTime(new Date())
                            .build();
                    tableInfoService.saveOrUpdate(build, new LambdaUpdateWrapper<>(TableInfo.class) {{
                        eq(TableInfo::getTableName, build.getTableName());
                        eq(TableInfo::getBasicId, etlDataScore.getId());
                    }});
                    //根据数据库id和表名查询数据
                    TableInfo table = tableInfoService.selectTableInfoByName(build);


                    ExecutorService threadPool = Executors.newCachedThreadPool();

                    threadPool.submit(() -> {
                        syncData(conn, etlDataScore.getDatabaseName(), table);
                    });
                    preparedStatement.close();
                }
                conn.close();
                mysqlPool.closeConn();
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

        return 1;
    }


    private void syncData(Connection conn, String databaseName, TableInfo table) {
        ExecutorService threadPool = Executors.newCachedThreadPool();
        PreparedStatement ps = null;
        try {
            ps = conn.prepareStatement(SELECTFIELD+"WHERE  \n" +
                    "TABLE_SCHEMA = '" + databaseName + "'   \n" +
                            "AND TABLE_NAME = '" + table.getTableName() + "'");
            ResultSet resultSet = ps.executeQuery();
            while (resultSet.next()) {
                String columnName = String.valueOf(resultSet.getString(1));
                String columnComment = String.valueOf(resultSet.getObject(2));
                String columnKey = String.valueOf(resultSet.getObject(3));
                String end = String.valueOf(resultSet.getObject(4));
                String dataType = String.valueOf(resultSet.getObject(5));
                String columnType = String.valueOf(resultSet.getObject(6));
                String characterMaximumLength = String.valueOf(resultSet.getInt(7));
                String numericScale = String.valueOf(resultSet.getInt(8));
                String isNullable = String.valueOf(resultSet.getObject(9));
                String columnDefault = String.valueOf(resultSet.getObject(10));

                Structure build = Structure.builder()
                        .tableId(table.getId())
                        .columnName(String.valueOf(columnName))
                        .columnRemark(columnComment)
                        .isPrimary("是".equals(columnKey) ? "Y" : "N")
                        .javaType(end)
                        .columnType(columnType)
                        .columnLength(characterMaximumLength)
                        .columnDecimals(numericScale)
                        .isNull("YES".equals(isNullable) ? "Y" : "N")
                        .defaultValue(columnDefault)
                        .build();

                threadPool.submit(() -> {
                    structureService.saveOrUpdate(build, new LambdaUpdateWrapper<Structure>() {{
                        eq(Structure::getTableId, build.getTableId());
                        eq(Structure::getColumnName, build.getColumnName());
                        eq(Structure::getColumnRemark, build.getColumnRemark());
                    }});
                });

            }
            threadPool.shutdown();
            ps.close();

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

}



